import uuid
import unittest
import base64
import urlparse

from httmock import urlmatch, HTTMock
from pyenketo import Enketo, Http404


def check_token(request):
    auth_type, value = tuple(request.headers['Authorization'].split(" "))
    token = base64.b64decode(value)
    if token != 'abc:':
        return False, {
            'status_code': 401,
            'content': '{"code": "401", "message": "Access denied: invalid'
                       ' authentication token."}'
        }
    else:
        return True, None


@urlmatch(netloc=r'(.*\.)?test.enketo\.org$', path='/api_v1/survey')
def get_survey_mock(url, request):
    # if the token is not valid, return a 401
    token_valid, response = check_token(request)
    if not token_valid:
        return response

    # check for non-existent survey
    server_url = urlparse.unquote(urlparse.urlparse(url.query).path)
    path = urlparse.urlparse(
        dict(urlparse.parse_qsl(server_url))['server_url']).path
    if path == '/notexist':
        return {
            'status_code': 404,
            'content': '{"message": "Form does not exist"}'
        }

    return '{"code": "200", "url": "https://cz2pj.enketo.org/webform"}'


@urlmatch(netloc=r'(.*\.)?test.enketo\.org$', path='/api_v1/instance')
def get_edit_url_mock(url, request):
    # if the token is not valid, return a 401
    token_valid, response = check_token(request)
    if not token_valid:
        return response

    return {
        'status_code': 201,
        'content': '{"code": "201", "edit_url": "https://cz2pj-0.enketo.org/webform/edit?instance_id=2"}'
    }


class TestPyEnketo(unittest.TestCase):
    def test_get_survey_url(self):
        enketo = Enketo()
        enketo.configure(
            ENKETO_URL='https://test.enketo.org/', API_TOKEN='abc')

        with HTTMock(get_survey_mock):
            url = enketo.get_survey_url(
                'https://testserver.com/bob', 'widgets')
            self.assertEqual(url, "https://cz2pj.enketo.org/webform")

    def test_get_survey_notexist(self):
        enketo = Enketo()
        enketo.configure(
            ENKETO_URL='https://test.enketo.org/', API_TOKEN='abc')
        with HTTMock(get_survey_mock):
            self.assertRaises(
                Http404,
                enketo.get_survey_url,
                'https://testserver.com/notexist',
                'widgets')

    def test_get_edit_url(self):
        enketo = Enketo()
        enketo.configure(
            ENKETO_URL='https://test.enketo.org/', API_TOKEN='abc')
        xml_instance = '<?xml version=\'1.0\' ?><clinic_registration id="clinic_registration"><formhub><uuid>73242968f5754dc49c38463af658f3d2</uuid></formhub><user_id>0</user_id><clinic_name>Test clinic</clinic_name><meta><instanceID>uuid:ec5ce15e-5a0a-4246-93fe-acf60ef69bf2</instanceID></meta></clinic_registration>'
        instance_id = uuid.uuid4()
        return_url = 'https://testserver.com/bob/1'
        with HTTMock(get_edit_url_mock):
            enketo.get_edit_url(
                'https://testserver.com/bob',
                'widgets',
                xml_instance,
                instance_id,
                return_url)



import requests
import json
from urlparse import urljoin


class EnketoException(Exception):
    def __init__(self, code, message):
        self.code = code
        self.message = message


class Http400(EnketoException):
    code = 400
    message = "bad request"


class Http401(EnketoException):
    code = 401
    message = "Access denied: invalid authentication token."


class Http403(EnketoException):
    code = 403
    message = "Authentication succeeded but account inactive or quota filled" \
              " up."


class Http404(EnketoException):
    code = 404
    message = "Resource was not found in the database"


class Http405(EnketoException):
    code = 405
    message = "Request not allowed"


class Http410(EnketoException):
    code = 410
    message = "Endpoint deprecated"


status_code_exception_mapping = {
    (400, Http400),
    (401, Http401),
    (403, Http403),
    (404, Http404),
    (405, Http405),
    (410, Http410)
}


def exception_for_response_code(status_code):
    excs = filter(
        lambda mapping: mapping[0] == status_code,
        status_code_exception_mapping)
    code, klass = excs[0]
    return klass


class Enketo(object):
    ENKETO_URL = 'https://enketo.org'
    API_PATH = '/api_v1'
    SURVEY_PATH = '/survey'
    INSTANCE_PATH = '/instance'
    API_TOKEN = None

    def configure(self, ENKETO_URL, API_TOKEN):
        self.ENKETO_URL = ENKETO_URL
        self.API_TOKEN = API_TOKEN

    def get_survey_url(self, server_url, form_id):
        payload = {'server_url': server_url, 'form_id': form_id}
        response = requests.get(
            urljoin(
                self.ENKETO_URL,
                "{}{}".format(self.API_PATH, self.SURVEY_PATH)),
            auth=(self.API_TOKEN, ''),
            params=payload)
        content = json.loads(response.content)
        if response.status_code != 200:
            raise exception_for_response_code(response.status_code)(
                response.status_code, content['message'])
        else:
            return content['url']

    def get_edit_url(
            self, server_url, form_id, xml_instance, instance_id, return_url):
        payload = {
            'server_url': server_url,
            'form_id': form_id,
            'instance': xml_instance,
            'instance_id': instance_id,
            'return_url': return_url
        }
        response = requests.post(
            urljoin(
                self.ENKETO_URL,
                "{}{}".format(self.API_PATH, self.INSTANCE_PATH)),
            auth=(self.API_TOKEN, ''),
            data=payload)
        content = json.loads(response.content)
        if response.status_code != 201:
            raise exception_for_response_code(response.status_code)(
                response.status_code, content['message'])
        else:
            return content['edit_url']